import matplotlib.pyplot as plt
import numpy as np
import serial
import time
from mpl_toolkits.axes_grid1 import make_axes_locatable

ser = serial.Serial('/dev/cu.wchusbserial110', 115200)
ser.flushInput()

ser_bytes = ser.readline()

# preallocating variables
norm_pix = []
cal_vec = []
kk = 0
cal_size = 10 # size of calibration
cal_pix = []
time_prev = time.time() # time for analyzing time between plot updates

def handle_close(evt):
    raise SystemExit('Closed figure, exit program.')

def get_data():
    global data, ser
    line = ser.readline().decode().strip()
    line = line.rstrip(line[-1])
    temps = [float(t.strip() or 0) for t in line.split(",")]
    data = np.array(temps).reshape((8, 8))
    return data

plt.ion()

try:
    while(1):
        norm_pix = get_data()
        if kk == 0:
            print("Sensor should have clear path to calibrate against environment")
            graph = plt.imshow(np.reshape(np.repeat(0,64),(8,8)), cmap=plt.cm.viridis , interpolation='lanczos')
            plt.colorbar()
            plt.clim(1, 8)  # can set these limits to desired range or min/max of current sensor reading
            plt.show()
            plt.pause(0.0001)
        if kk < cal_size + 1:
            kk += 1
        if kk == 1:
            cal_vec = norm_pix
            continue
        elif kk <= cal_size:
            for xx in range(0, len(norm_pix)):
                cal_vec[xx] += norm_pix[xx]
                if kk == cal_size:
                    cal_vec[xx] = cal_vec[xx] / cal_size
            continue
        else:
            cal_pix = np.subtract(np.array(norm_pix), np.array(cal_vec))
            if cal_pix.min() < 0:
                for y in range(0, len(cal_pix)):
                    cal_pix[y] += abs(cal_pix.min())

        # Moving Pixel Plot #
        graph.set_data(np.reshape(cal_pix,(8,8)))  # updates heat map in 'real-time'
        #plt.clim(cal_pix.min(), cal_pix.max())
        plt.show()  # plots updated heat map
        plt.pause(0.0001)
        cal_pix = []  # off-load variable for next reading
        print(time.time() - time_prev)  # prints out time between plot updates
        time_prev = time.time()

        #print(get_data())

except KeyboardInterrupt:
        print("CTRL-C: Program Stopping via Keyboard Interrupt...")
        exit()

finally:
        print("Exiting Loop")